/*****************************************************************************/
/*   INVFFTC.C                                                               */
/*   Checks the real inverse FFT code for a 64-point FFT run on a C3x device.*/
/*   It can be extended to different sizes by changing the global constants  */
/*   defined below.                                                          */
/*   For FFT of size below 1024 points or equal to 1024 points , FFT input   */
/*   and output data can be put in internal RAM , same location. Also the    */
/*   twiddle factor table and FFT code can be put in internal RAM.           */
/*   If FFT size is over 1024 points, please watch the memory allocation.    */
/*                                                                           */
/*   In this example, the IFFT input is a single real positive value , and   */
/*   we expect to get a cosine waveform as the IFFT output.                  */
/*****************************************************************************/
#include "values.h"
#include "math.h"
/*===========================================================================*/
/* GLOBAL CONSTANTS                                                          */
/*===========================================================================*/

#define FFT_SIZE               64
#define HALF_FFT_SIZE          32
#define LOG2_SIZE               6
#define BITREV                  1      /*  Bit reverse will be performed   */

/*===========================================================================*/
/* GLOBAL VARIABLES                                                          */
/*===========================================================================*/
int i;
float theta;
volatile int *ctrlreg = (volatile int *) 0x808064; /* primary bus register */
volatile float *FFTBufferIn = (volatile float *) 0x809C00; /* RAM BLOCK 1 */
volatile float *FFTBufferOut= (volatile float *) 0x809C00; /* RAM BLOCK 1 */
volatile float *SinTable = (volatile float *) 0x809A00; /* RAM BLOCK 0 */

/*===========================================================================*/
/* MAIN()                                                                    */
/*===========================================================================*/
void main()
{ asm ("   OR 0800h,ST");    /* cache is on */
  *ctrlreg = 0x0;            /* no wait state */
  theta = 2*PI/FFT_SIZE;
  for (i=0;i<HALF_FFT_SIZE;i++)   /* fill sin table in memory */
     SinTable[i]=sin(i*theta);

  for (i=0;i<FFT_SIZE;i++)	/* fill fft input table */
     FFTBufferIn[i]=0.0;
  FFTBufferIn[4]=32.0;

  ifft_rl( FFT_SIZE, LOG2_SIZE, FFTBufferIn, FFTBufferOut,SinTable,BITREV);

}

