/*****************************************************************************/
/*   R2DIT.C                                                                 */
/*   Checks the complex FFT code for a 64-point FFT run on a C3x device.     */
/*   It can be extended to different sizes by changing the global constants  */
/*   defined below.                                                          */
/*   For FFT of size below 1024 points or equal to 1024 points , FFT input   */
/*   data can be put in internal RAM .                                       */
/*   If FFT size is over 1024 points, please watch the memory allocation.    */
/*                                                                           */
/*   In this example, the FFT input is the one defined in TI DSP Application */
/*   Book Number 3.                                                          */
/*****************************************************************************/

#include "math.h"
#include "values.h"
/*===========================================================================*/
/* GLOBAL CONSTANTS                                                          */
/*===========================================================================*/

#define FFT_SIZE               64
#define LOG2_SIZE               6

/*===========================================================================*/
/* GLOBAL VARIABLES                                                          */
/*===========================================================================*/
float theta;
int i;
volatile int *ctrlreg = (volatile int *) 0x808064; /* primary bus register */
volatile float *FFTBufferIn = (volatile float *) 0x809C00; /* RAM BLOCK 1 */
volatile float *FFTBufferOut = (volatile float *) 0x809D00; /* RAM BLOCK 1 */
volatile float *SinTable = (volatile float *) 0x809A00; /* RAM BLOCK 0 */
int halfsize = FFT_SIZE/2;
int quartsize = FFT_SIZE/4;
int quartsize_minus2 = (FFT_SIZE/4)-2;
int octsize_minus2 = (FFT_SIZE/8)-2;

float FFTIn[128] = {
    0.2113,
    0.0000,
    0.0824,
    0.0000,
    0.7599,
    0.0000,
    0.0087,
    0.0000,
    0.8096,
    0.0000,
    0.8474,
    0.0000,
    0.4524,
    0.0000,
    0.8075,
    0.0000,
    0.4832,
    0.0000,
    0.6135,
    0.0000,
    0.2749,
    0.0000,
    0.8807,
    0.0000,
    0.6538,
    0.0000,
    0.4899,
    0.0000,
    0.7741,
    0.0000,
    0.9626,
    0.0000,
    0.9933,
    0.0000,
    0.8360,
    0.0000,
    0.7469,
    0.0000,
    0.0378,
    0.0000,
    0.4237,
    0.0000,
    0.2613,
    0.0000,
    0.2403,
    0.0000,
    0.3405,
    0.0000,
    0.1167,
    0.0000,
    0.6250,
    0.0000,
    0.5510,
    0.0000,
    0.3550,
    0.0000,
    0.4943,
    0.0000,
    0.0365,
    0.0000,
    0.2260,
    0.0000,
    0.8159,
    0.0000,
    0.2284,
    0.0000,
    0.8553,
    0.0000,
    0.0621,
    0.0000,
    0.7075,
    0.0000,
    0.2408,
    0.0000,
    0.6907,
    0.0000,
    0.1062,
    0.0000,
    0.2640,
    0.0000,
    0.7034,
    0.0000,
    0.4021,
    0.0000,
    0.6553,
    0.0000,
    0.9700,
    0.0000,
    0.0380,
    0.0000,
    0.0988,
    0.0000,
    0.2560,
    0.0000,
    0.5598,
    0.0000,
    0.9166,
    0.0000,
    0.1402,
    0.0000,
    0.7054,
    0.0000,
    0.0178,
    0.0000,
    0.2611,
    0.0000,
    0.1358,
    0.0000,
    0.0503,
    0.0000,
    0.5782,
    0.0000,
    0.2432,
    0.0000,
    0.9448,
    0.0000,
    0.5876,
    0.0000,
    0.7256,
    0.0000,
    0.2849,
    0.0000,
    0.6767,
    0.0000,
    0.8642,
    0.0000,
    0.1943,
    0.0000};

/*===========================================================================*/
/* MAIN()                                                                    */
/*===========================================================================*/
void main()
{ asm ("   OR 0800h,ST");    /* cache is on */
  *ctrlreg = 0x0; /* no wait state */
  theta = 2*PI/FFT_SIZE;
  for (i=0;i<quartsize;i++)   /* fill sin table in memory */
   { SinTable[1+(2*i)]=sin(i*theta);
     SinTable[2*i]=cos(i*theta);}
  bitrev(SinTable,quartsize);

  for (i=0;i<FFT_SIZE*2;i++)      /* fill fft input table */
     FFTBufferIn[i]=FFTIn[i];

  ffft_r2( FFT_SIZE, LOG2_SIZE, FFTBufferIn,FFTBufferOut,SinTable);       }


