/*****************************************************************************/
/*   R2IDIT.C                                                                */
/*   Checks the complex inverse FFT code for a 64-point FFT run on           */
/*   a C3x device.                                                           */
/*   It can be extended to different sizes by changing the global constants  */
/*   defined below.                                                          */
/*   For FFT of size below 1024 points or equal to 1024 points , FFT input   */
/*   data can be put in internal RAM .                                       */
/*   If FFT size is over 1024 points, please watch the memory allocation.    */
/*                                                                           */
/*   In this example, the FFT output should be a sine waveform.              */
/*****************************************************************************/

#include "math.h"
#include "values.h"
/*===========================================================================*/
/* GLOBAL CONSTANTS                                                          */
/*===========================================================================*/

#define FFT_SIZE                64
#define HALF_FFT_SIZE           32
#define LOG2_SIZE                6

/*===========================================================================*/
/* GLOBAL VARIABLES                                                          */
/*===========================================================================*/
int i;
float theta;
volatile int *ctrlreg = (volatile int *) 0x808064; /* primary bus register */
volatile float *FFTBufferIn = (volatile float *) 0x809C00; /* RAM BLOCK 1 */
volatile float *FFTBufferOut= (volatile float *) 0x809D00; /* RAM BLOCK 1 */
volatile float *SinTable = (volatile float *) 0x809B00; /* RAM BLOCK 0 */


int halfsize = FFT_SIZE/2;
int quartsize = FFT_SIZE/4;
int quartsize_minus2 = (FFT_SIZE/4)-2;
int octsize_minus2 = (FFT_SIZE/8)-2;

		     /*  64 values theorical values of this FFT */
/*===========================================================================*/
/* MAIN()                                                                    */
/*===========================================================================*/
void main()
{ asm ("   OR 0800h,ST");    /* cache is on */
  *ctrlreg = 0x0; /* no wait state */
  theta = 2*PI/FFT_SIZE;

  for (i=0;i<quartsize;i++)   /* fill sin table in memory */
   { SinTable[1+(2*i)]=sin(i*theta);
     SinTable[2*i]=cos(i*theta);}
  bitrev(SinTable,quartsize);

  for (i=0;i<FFT_SIZE*2;i++)      /* fill fft input table */
     FFTBufferIn[i]=0.0;
  FFTBufferIn[5]=-0.5*FFT_SIZE;
  FFTBufferIn[(2*FFT_SIZE)-3]=0.5*FFT_SIZE;

  ifft_r2( FFT_SIZE, LOG2_SIZE, FFTBufferIn,FFTBufferOut,SinTable);
  }
