/**************************************************************
  
                 ieeeconv.c
  
                 staff
  
                 12-02-91
  
           (C) Texas Instruments Inc., 1992 
  
           Refer to the file 'license.txt' included with this 
           this package for usage and license information. 
  
*************************************************************/

/*****************************************************************************/
/* THE TMS320C30 DOES NOT USE THE IEEE STANDARD REPRESENTATION FOR FLOATING  */
/* POINT NUMBERS.  THE TWO REPRESENTATIONS MUST BE CONVERTED BY THE TMS320C30*/
/* FOR THEM TO BE UNDERSTOOD.                                                */
/*****************************************************************************/

struct c30float
{
    signed   int mantissa:23;
    unsigned int sign:1;
    signed   int exponent:8;
};

struct ieeefloat
{
    unsigned int mantissa:23;
    unsigned int exponent:8;
    unsigned int sign:1;
};

float fmieee(int i)       /* CONVERSION FROM IEEE FORMAT TO TMS320C30 FORMAT */
{
    union ieee_union            /* IEEE CAN BE TREATED AS AN INT OR A STRUCT */
    {
	unsigned int       in;
	struct   ieeefloat str;
    } ieee;
    union c30_union            /* C30 CAN BE TREATED AS A FLOAT OR A STRUCT */
    {
	float           flt;
	struct c30float str;
    } c30;

    ieee.in = i;
    c30.str.mantissa = ieee.str.mantissa;    /* THE MANTISSAS ARE IDENTICAL */
    /* THE EXPONENT IN THE IEEE FLOATING POINT FORMAT IS 127 ADDED TO THE   */
    /* ACTUAL VALUE.                                                        */
    c30.str.exponent = (ieee.str.exponent == 0)
		       ? -128 :	ieee.str.exponent - 127;
    c30.str.sign = 0;                        /* MAKE THE SIGN POSITIVE      */
    /* IF IT IS NEGATIVE, NEGATE THE C30 FLOATING POINT VALUE.              */
    if (ieee.str.sign) c30.flt = -c30.flt;
    return (c30.flt);
}

unsigned int toieee(float x)       /*CONVERSION FROM TMS320C30 TO IEEE FORMAT*/
{
    union ieee_union            /* IEEE CAN BE TREATED AS AN INT OR A STRUCT */
    {
	unsigned int       in;
	struct   ieeefloat str;
    } ieee;
    union c30_union             /* C30 CAN BE TREATED AS A FLOAT OR A STRUCT */
    {
	float           flt;
	struct c30float str;
    } c30;
    c30.flt = x;
    ieee.str.sign = c30.str.sign;    /* THE SIGN IS THE SAME IN BOTH FORMATS */
/* IF C30 IS NEGATIVE, NEGATE IT (MAKE IT POSITIVE)                          */
    if (c30.str.sign) c30.flt = -c30.flt;
    ieee.str.exponent = (c30.str.exponent == -128) ? 0 :        /* SEE ABOVE */
			 c30.str.exponent + 127;
    ieee.str.mantissa = c30.str.mantissa;      /* THE MANTISSAS ARE THE SAME */
    return (ieee.in);
}


